MODULE Pentium;
IMPORT IO;

(* Pentium bug detection, SWAG translation, no copyrights, I translated
    it for a friend, and why not include it here?

    My Cyrix has no bug:-)

        Translation to M2 by Marco van de Voort in 1996.*)

PROCEDURE TestPentium; (* Compile with coprocessor on, no emulation! *)

(*
According to a message I just saw on Usenet, many Pentiums have a bug in
their implementation of the FDIV (floating point divide) instruction.
Supposedly the following program prints the value 256.0000, rather than
0.0000, on one of these:
*)

VAR
  x,y,z : LONGREAL;
BEGIN
    x := 4195835.0;
    y := 3145727.0;
    z := x - (x/y)*y;
    IO.WrStr(" z = ");
    IO.WrLngReal(z,0,4);
    IO.WrStr(' (should be approx. 0.0000, small deviations like 1E-14 are allowed)');
    IO.WrLn;
    IO.WrLn;
END TestPentium;

(*
From: bobs@dragons.nest.nl (Bob Swart)

On the BPASCAL forum on CompuServe, DJ Murdoch [71631,122] reported something
that isn't a Borland bug, but looks as though it'll affect BP programs when
they run on a Pentium:

64.  For certain very rare pairs x and y, a $N+ division x/y will only be
accurate to about 4 decimal digits when calculated on a Pentium produced before
Fall 1994.

Dr. Bob's further detailed analysis resulted in the following function which
will generate 11 series of infinite "buggy" numbers for which the following
does not hold on a Pentium chip (a reciprocal of a reciprocal):

 x = 1/1/x

Try it on a Pentium with for example the magic number... 357914069

I also found that a power of 2 times any number that goes wrong also goes wrong
(i.e. X, 2*X, 4*X, 8*X, etc).

Also, once you've found a "wrong number" (and the serie), you can start a new
serie by multiplying with 4 and adding 383 (= 256 + 128 - 1).

Below follows a general function that is able to produce 11 series of (just
about infinite) numbers that generate incorrect results. Note that the initial
difference (for the first digit in a serie) is 21 1/3, and the difference
increases with the same power of two as the original number...
*)

PROCEDURE DrBob5;
(* Digits found in one night (source for analysis):

   357914069, 715828138, 1431656276,
   1431655893 (new series = 4 * X - 383 ) *)

  PROCEDURE PentiumBug(Serie,Index: CARDINAL): LONGREAL;
  (* Serie max = 11 *)
  CONST Magic = 357914069;
        Factor= 256 + 128 - 1;

  VAR Tmp: LONGREAL;
      i: CARDINAL;
  BEGIN
    Tmp := LONGREAL(Magic);
    FOR i:=2 TO Serie DO
     Tmp := 4.0 * Tmp - LONGREAL(Factor);
     END;
    FOR i:=2 TO Index DO
     Tmp := Tmp + Tmp;
     END;
    RETURN Tmp;
  END PentiumBug;

VAR i,j: CARDINAL;
    x,y,z: LONGREAL;
BEGIN
  FOR i:=1 TO 11 DO
    FOR j:=1 TO 16 DO
      x := PentiumBug(i,j);
      y := 1.0 / x;
      z := 1.0 / y;
    (* z should be x, but isn't on buggy ones ... *)
      IO.WrStr('x = '); IO.WrLngReal(x,9,9);
      IO.WrStr(' 1/x ='); IO.WrLngReal(y,9,9);
      IO.WrStr(' 1/1/x = ');IO.WrLngReal(z,9,9);
      IO.WrStr(' diff '); IO.WrLngReal(x-z,4,0);
      IO.WrLn;
    END;
    IO.WrLn;
   END;
END DrBob5;
(*
The bug is in the FDIV instruction, which for about 1 in 10^10 pairs of
randomly chosen divisors suffers a catastrophic loss of precision, ending up
accurate to 4 to 6 digits instead of the usual 18.  Apparently Intel chose a
division algorithm which is fast but which isn't 100% guaranteed to work.

I still don't know why the difference is 0 again starting with the 12th serie,
but I'm sure given enough time I can reproduce the exact buggy FDIV algorithm
Intel uses...
*)

BEGIN
 TestPentium;
 DrBob5;
END Pentium.
