MODULE Poetry;
(* I wanted to do some beautifying of identifiers in Pas2Mod eand I searched for
a routine which could split a word into syllables. Somebody on FIDO-net
pointed me to this SWAG post. It wasn't any good. It can only count the
syllables in a word, can't determine where one syllable ends, and where
the other starts.

No copyrights, "integral" (as far as possible, the original was a bit clumsy
sometimes, it even calculated the conson charset runtime with a FOR statement)
translation. I only tested it very briefly. If you intend to use
it on full scale texts, then recheck!

Another problem was that all functions returned STRINGS, which isn't possible
in M2. Changed it to procedures, resembling Str procedures

Of course, this program was designed for English. Entering Dutch sentences gave
almost always too little syllables.

*)

FROM Str IMPORT Slice,Length,Caps,CHARSET,Append,Pos,Copy;

IMPORT IO;

CONST
  Vowels = CHARSET{'A', 'E', 'I', 'O', 'U', 'Y'};
  Alphabet= CHARSET{'A'..'Z'};
  Conson =Vowels/Alphabet;

PROCEDURE RightStr( VAR S : ARRAY OF CHAR;S2 : ARRAY OF CHAR; Index : BYTE );
BEGIN
  Slice(S,S2,Length(S2)-CARDINAL(Index),CARDINAL(Index));
END RightStr;

(* This function returns the number of words within a given string. *)
PROCEDURE NumWords( S : ARRAY OF CHAR ) : CARDINAL;
VAR
  Words,
  Count : CARDINAL;

BEGIN
  Words := 0;
  Count := 0;
  Caps(S);

  WHILE Count < Length( S ) DO
    INC( Count );

    IF S[Count] IN Alphabet THEN
      INC( Words );
      WHILE (S[Count] IN Alphabet) AND (Count < Length( S )) DO
        INC( Count );
        END;
    END;
  END;
 RETURN Words;
 END  NumWords;

(* This function will return a word (1st, 2nd, 3rd, etc) from a sentence.
  Note that it converts the word to uppercase. *)
PROCEDURE GetWord(VAR RetS:ARRAY OF CHAR; S : ARRAY OF CHAR; WhichWord : CARDINAL );

VAR
  WC,
  Count : CARDINAL;

BEGIN
  WC    := 0;
  Count := 0;
  RetS[0]:=CHR(0);

  IF WhichWord > NumWords( S ) THEN
    RetS[0] := CHR(0);
    RETURN;
    END;
  Caps(S);

  WHILE Count < Length( S ) DO

    IF S[Count] IN Alphabet THEN
      INC( WC );

      IF WC = WhichWord THEN
        WHILE (S[Count] IN Alphabet) AND (Count <= Length( S )) DO
          Append(RetS,S[Count]);
          INC( Count );
          END
      ELSE
        WHILE (S[Count] IN Alphabet) AND (Count <= Length( S )) DO
          INC( Count );
          END;
         END;
      END;
          INC( Count );
   END;
END GetWord;

(* This function will return the number of syllables in a given word.
  It is in no way a fool-proof function, but will work for a lot of words. *)
PROCEDURE WORD_Syll( S : ARRAY OF CHAR ) : CARDINAL;
VAR
  Count,
  SylCount : CARDINAL;
  SmStr : ARRAY[0..10] OF CHAR;

BEGIN
  (* No syllables! *)
  SylCount := 0;
  Caps(S);

  (* If it starts with a vowel, there is a good chance an extra syllable is in
    there. *)
  IF (S[1] IN Vowels) AND (Length( S ) > 3) THEN
    SylCount := 1;
    END;

  IF (Pos(S,'IO' ) # MAX(CARDINAL)) THEN
    INC( SylCount );
    END;
  IF (Pos(S, 'EO' ) # MAX(CARDINAL)) THEN
    INC( SylCount );
    END;
  IF (Pos(S, 'IA' ) # MAX(CARDINAL)) THEN
    INC( SylCount );
    END;
  IF (Pos(S, 'ISM' ) # MAX(CARDINAL)) THEN
    INC( SylCount );
    END;
  IF ( Pos( S,'UA' ) # MAX(CARDINAL)) AND ( Pos(S, 'QUA' ) =MAX(CARDINAL)) THEN
    INC( SylCount );
    END;
  IF (Pos( S,'TION' ) OR (Pos( S,'SION')) # MAX(CARDINAL)) THEN
    DEC( SylCount );
    END;

  FOR Count := 1 TO (Length( S ) - 1) DO
    IF (S[Count] IN Conson) AND (S[Count + 1] IN Vowels) THEN
      INC( SylCount );
      END;
     END;

  RightStr(SmStr,S,3);
  IF ((S[Length( S )-1] = 'E') AND (Pos( SmStr,'BLE') = MAX(CARDINAL))) AND
     ((Pos(SmStr, 'IE') = MAX(CARDINAL)) AND
     (Pos(SmStr, 'TLE') = MAX(CARDINAL))) THEN
    DEC( SylCount );
    END;

  (* A word must have at least 1 syllable!! *)
  IF SylCount < 1 THEN
    SylCount := 1;
    END;
 RETURN SylCount;
END WORD_Syll;

(* This function will count the number of syllables in a given sentence. *) 
PROCEDURE Sent_Syll( S : ARRAY OF CHAR ) : CARDINAL;
VAR
  Count,
  SylCount : CARDINAL;
  Temp     : ARRAY[0..100] OF CHAR;

BEGIN
  SylCount := 0;
  FOR Count := 1 TO NumWords( S ) DO
   GetWord(Temp,S,Count);
   INC( SylCount, WORD_Syll( Temp));
   END;
 RETURN SylCount;
END Sent_Syll;



CONST
  TEST1 = 'These are a few interesting functions, man.';
  TEST2 = 'antidisestablishmentarianism';

VAR
  AWord : ARRAY[0..100] OF CHAR;
  TEST : ARRAY[0..100] OF CHAR;

PROCEDURE DoSyl;

BEGIN
  GetWord( AWord,TEST, NumWords( TEST ) );

  IO.WrStr( 'The last word is : '); IO.WrStr(AWord); IO.WrLn;
  IO.WrStr( '# OF syllables   : '); IO.WrCard( WORD_Syll( AWord ) ,0); IO.WrLn;
  IO.WrStr( 'Total # of words : '); IO.WrCard( NumWords( TEST ) ,0);   IO.WrLn;
  IO.WrStr( 'Total syllables  : '); IO.WrCard( Sent_Syll( TEST ),0 );  IO.WrLn;
END DoSyl;

BEGIN
 Copy(TEST,TEST1);
 REPEAT
  DoSyl;
  IO.RdStr(TEST)
 UNTIL TEST[0]=CHR(0);

END Poetry.
