MODULE SubComp;
(* The resource compiler for PullDown,XtdLib package,
                              (C) by Marco van de Voort 1997
 Compiles .SRC files to .DEF files for use by PullDown. Since .SRC files export
 constants to the Modula-2 mainmodule, one always has to create a .DEF file as
 an intermediate.
 I could put the rest of the data in a binary file, but I chose to put the
 rest in  the .DEF file too, to keep the number of files down.

Ugly layout and interface, but making it more userfriendly means more XtdLib
procedures, which I don't want right now. *)

FROM Storage IMPORT ALLOCATE,DEALLOCATE,HeapTotalAvail,MainHeap;
FROM Lib     IMPORT ParamCount,ParamStr,EnableBreakCheck;
FROM Str     IMPORT CharPos,Delete,Length,Copy,Append,ItemS,CHARSET,
                    Caps,Match,Compare;
FROM IO      IMPORT WrStr,WrLn,WrCard;
FROM ELib    IMPORT KillChar,KillBChar; (* XtdLib!, substitutes in ModAsm*)

IMPORT FIO,IO;

(*
Substitutes for XtdLib procedures.

PROCEDURE KillChar(VAR InStr : ARRAY OF CHAR;C:CHARSET);

BEGIN
 WHILE (InStr[0] IN C) DO Delete(InStr,0,1); END;
END KillChar;

PROCEDURE KillBChar(VAR InSt:ARRAY OF CHAR;C :CHARSET);

BEGIN
 LOOP
  IF (InSt[Length(InSt)-1] IN C) AND (Length(InSt)>0) THEN
   Delete(InSt,Length(InSt)-1,1);
  ELSE
   EXIT;
   END;
  END;
END KillBChar;
*)

      (* These constants are copied from PullDown.MOD, internal constants,
      used for a kind of uuencoding. *)

CONST NrO     =25;                (* Maximal number of submenu's *)
      Min    = 40;                (* lowest ASCII code used *)
      Divver = 256-Min;           (* Total # of ASCII codes *)
      DvD    = Divver*(Divver-1); (* Top "Divver" 2-bytes codes reserved for
                                     special things. DvD means
                                     Divver * Divver, the dutch word for
                                     multiplication starts with a "v"
                                     Note that the original definition
                                     was different! DvDm1 is however even more
                                     cryptic*)

TYPE       Line_Pnt  = POINTER TO Line_Rec;

           Line_Rec=RECORD                 (* The mainrecord type. *)
            Next     : Line_Pnt;              (* Next Pointer*)
            CallSign : CARDINAL;        (* constant-code *)
            Label,                      (* Label if not sub *)
            Name     : ARRAY[0..25] OF CHAR; (*Name*)
            END;
           Str      = ARRAY [0..99] OF CHAR;

           MainData = RECORD
                       Path,Name :  ARRAY[0..99] OF CHAR;    (* Path and name of
                                                              .DEF to create*)
                       OBJName   :  ARRAY[0..40] OF CHAR;    (* Name of Object to create*)
                       Dat       :  ARRAY[1..NrO] OF Line_Pnt;     (* Pointers to submenus*)
                       Labels,                               (* Last Label used *)
                       CurHigh,                              (* Highest submenu nr used*)
                       Cur,                                  (* Current submenu nr *)
                       NameX     :  CARDINAL;                (* Nr of Topmenus*)
                       NameNr    :  ARRAY[1..NrO] OF CARDINAL; (* MenuNr's of Top
                                                                  menus*)
                       Names     :  ARRAY[1..NrO] OF Str;    (* Names of Topmenus*)
                       Sub,                                  (* Submenu or PullDown?*)
                       First     :  BOOLEAN;                 (* First object?
                                                                i.o.w. should I write a
                                                                DEFINITION line*)

                       END;
           MainP    = POINTER TO MainData;

PROCEDURE Init(R:MainP);

VAR A: CARDINAL;

BEGIN
  FOR A:= 1 TO NrO DO
   R^.Dat[A]:=NIL;
   END;
  R^.Cur:=0;
  R^.CurHigh:=0;
  R^.Labels:=1;
  R^.NameX:=0;
END Init;

PROCEDURE KillPnt(R:Line_Pnt);

VAR  Cur,Cur2 : Line_Pnt;

BEGIN
 Cur:=R;
 LOOP
  Cur2:=Cur;
  IF Cur^.Next=NIL THEN
   DISPOSE(Cur);
   EXIT
   END;
  Cur:=Cur^.Next;
  DISPOSE(Cur2);
  END;
END KillPnt;

PROCEDURE KillObject(R:MainP);

VAR A   : CARDINAL;

BEGIN
 FOR A:=1 TO R^.CurHigh DO
  IF R^.Dat[A]# NIL THEN
   KillPnt(R^.Dat[A]);
   END;
  END;
END KillObject;

PROCEDURE DisplayMem();
BEGIN
 IO.WrStr("Memory available : ");
 IO.WrLngCard(LONGCARD(HeapTotalAvail(MainHeap))*16,10); IO.WrLn;
END DisplayMem;

VAR F,F2        : FIO.File;             (* F= .SRC file, F2=.DEF file *)
    LineNr      : CARDINAL;             (* Line number, to report errors*)
    Buf,Buf2    : ARRAY[0..4095+FIO.BufferOverhead] OF  BYTE;

PROCEDURE Error(S:ARRAY OF CHAR);

BEGIN
 FIO.Close(F);
 FIO.Close(F2);
 WrStr("Error in Line "); WrCard(LineNr,0); WrLn; WrLn;
 WrStr(S); WrLn;WrLn;HALT;
END Error;

PROCEDURE MakeFile(R:MainP);
(* Create heading of object. If necessary include heading of definition file*)

VAR S : ARRAY[0..100] OF CHAR;

BEGIN
 Copy(S,R^.Path);
 Append(S,R^.Name);
 Append(S,".DEF");
 IF R^.First THEN
  F2:=FIO.Create(S);
  FIO.AssignBuffer(F2,Buf2);
  FIO.WrStr(F2,"DEFINITION MODULE ");
  FIO.WrStr(F2,R^.Name); FIO.WrStr(F2,";");
  FIO.WrLn(F2);FIO.WrLn(F2);
  FIO.WrStr(F2,"FROM PullDown IMPORT PullObject;");
  FIO.WrLn(F2);
  END;
END MakeFile;

VAR

    R           : MainP;                (* Major record, in pointer form,
                                          because originally I was
                                          planning to keep all objects in mem*)

PROCEDURE DoLabels(VAR L : CARDINAL);
(* Print all constants-labels of current object to .DEF file,
   subprocedure of SpoolMem *)

VAR A      : CARDINAL;
    TPoint : Line_Pnt;

BEGIN
 FOR A:=1 TO NrO DO
  IF R^.Dat[A] # NIL THEN
   TPoint:=R^.Dat[A];
   IF (TPoint^.CallSign=0) AND (TPoint^.Next # NIL) THEN
    TPoint:=TPoint^.Next;
    END;
   REPEAT
   IF TPoint^.Label[0] # CHR(0) THEN
    FIO.WrStr(F2," ");
    FIO.WrStrAdj(F2,TPoint^.Label,-15);
    FIO.WrStr(F2,"=");
    FIO.WrCard(F2,TPoint^.CallSign,6);
    FIO.WrStr(F2,";");
    INC(L);
    END;
   IF L=3 THEN
    FIO.WrLn(F2);
    L:=0;
    END;
   TPoint:=TPoint^.Next;
   UNTIL TPoint=NIL;
   END;
  END;
END DoLabels;

PROCEDURE SpoolMem(F2:FIO.File);
(* Write an object to .DEF file*)

VAR A      : CARDINAL;
    TPoint : Line_Pnt;
    TotBytes : CARDINAL;
    L        : CARDINAL;

BEGIN
 TotBytes:=0;

 (* Labels*)

 IF R^.First THEN
  FIO.WrLn(F2);
  FIO.WrStr(F2,"CONST ");
  FIO.WrLn(F2);
  END;
 L:=0;
 DoLabels(L);
 FIO.WrLn(F2);

 (* Write object, first part *)

 IF R^.Dat[1] # NIL THEN
  FIO.WrLn(F2);
  FIO.WrCharRep(F2," ",8);
  FIO.WrStr(F2,R^.OBJName);
  FIO.WrStr(F2,"_Dat");
  FIO.WrStr(F2,"=");
  FIO.WrLn(F2);
  END;


 FOR A:=1 TO NrO DO
  IF R^.Dat[A] # NIL THEN
   FIO.WrLn(F2) ;                    FIO.WrCharRep(F2,' ',25);
   FIO.WrStr(F2,"(*  SubMenu : ");   FIO.WrCard(F2,A,-3);
   FIO.WrStr(F2,"*)");               FIO.WrLn(F2) ;FIO.WrLn(F2) ;
   TPoint:=R^.Dat[A];
   IF (TPoint^.CallSign=0) AND (TPoint^.Next # NIL) THEN
    TPoint:=TPoint^.Next;
    END;
   REPEAT
    FIO.WrCharRep(F2," ",15);
    FIO.WrStr(F2,'"');
    FIO.WrStr(F2,TPoint^.Name);
    FIO.WrStr(F2,"'");
    FIO.WrStr(F2,'"');
    INC(TotBytes,Length(TPoint^.Name)+1);
    IF Length(TPoint^.Name)<21 THEN
     FIO.WrCharRep(F2," ",21-Length(TPoint^.Name));
     END;
     INC(TotBytes,2);
     FIO.WrStr(F2,"+ '");
     IF TPoint^.CallSign= DvD THEN
      FIO.WrStr(F2,CHR(255)+CHR(255));
     ELSE
      FIO.WrStr(F2,CHR(Min+TPoint^.CallSign DIV Divver));
      FIO.WrStr(F2,CHR(Min+TPoint^.CallSign MOD Divver));
      END;
     FIO.WrStr(F2,"'");
     FIO.WrStr(F2,"  (*");
     IF TPoint^.CallSign=DvD THEN
      IF TPoint^.Name[0]="|" THEN
       FIO.WrStr(F2,"͵");
      ELSE
       FIO.WrStr(F2," Menu Name");
       END;
     ELSE
      IF TPoint^.CallSign>DvD THEN
       FIO.WrStr(F2,"Submenu ");
       FIO.WrCard(F2,TPoint^.CallSign-DvD,-5);
      ELSE
       FIO.WrStrAdj(F2,TPoint^.Label,-10);
       END;
      END;
     FIO.WrStr(F2,"*)");
    TPoint:=TPoint^.Next;
    IF TPoint # NIL THEN
     FIO.WrStr(F2,"+");
     FIO.WrLn(F2);
     END;
    UNTIL TPoint=NIL
   END;
  IF R^.Dat[A] # NIL THEN
   INC (TotBytes,1);
   FIO.WrStr(F2,"+ CHR(0)");
   IF R^.Dat[A+1] # NIL THEN
    FIO.WrStr(F2,"+");
   ELSE
    FIO.WrStr(F2,";");
    END;
   FIO.WrLn(F2);
   END;
  END;
  FIO.WrLn(F2);                        FIO.WrCharRep(F2," ",40);
  FIO.WrStr(F2," (* SizeBlock =  ");   FIO.WrCard(F2,TotBytes+1,0);
  FIO.WrStr(F2," *)");                 FIO.WrLn(F2);
  FIO.WrLn(F2);

   (* Write second part (top-bar PullDown menu) if pullmenu object*)
  IF NOT R^.Sub THEN
   FIO.WrCharRep(F2," ",8);
   FIO.WrStr(F2,R^.OBJName);            FIO.WrStr(F2,"_Menu");
   FIO.WrStr(F2," ="+'"');              FIO.WrChar(F2,CHR(Min+R^.NameX));
   FOR A:=1 TO R^.NameX DO
    FIO.WrChar(F2,+CHR(Min+R^.NameNr[A]));
    END;
   FOR A := 1 TO R^.NameX DO
    FIO.WrChar(F2,"'");                 FIO.WrStr (F2,R^.Names[A]);
    IF (A MOD 5)=0 THEN
     FIO.WrStr(F2,'"+');                FIO.WrLn(F2);
     FIO.WrCharRep(F2," ",15);          FIO.WrChar(F2,'"');
     END;
    END;
   FIO.WrStr(F2,'";');
   FIO.WrLn(F2);FIO.WrLn(F2);
   END;
  FIO.WrLn(F2);

  (* Write third part, the actual identifier, which is passed to the
      module PullDown*)
  FIO.WrStr(F2,"  ");             FIO.WrStrAdj(F2,R^.OBJName,-15);
  FIO.WrChar(F2,"=");             FIO.WrStr(F2,"PullObject(");
  FIO.WrStr(F2,"ADR(");           FIO.WrStr(F2,R^.OBJName);
  FIO.WrStr(F2,"_Dat),");         FIO.WrStr(F2,"SIZE(");
  FIO.WrStr(F2,R^.OBJName);       FIO.WrStr(F2,"_Dat),");
  FIO.WrLn (F2);                  FIO.WrCharRep(F2," ",20);

     (* Address of structure 2 is NIL when submenu-object*)
  IF R^.Sub THEN
   FIO.WrStr(F2,"NIL);");
  ELSE
   FIO.WrStr(F2,"ADR(");           FIO.WrStr(F2,R^.OBJName);
   FIO.WrStr(F2,"_Menu));");
   END;
  FIO.WrLn (F2);                  FIO.WrLn (F2);
  IO.WrStr("(Object in memory)"); IO.WrLn;
  DisplayMem;
  KillObject(R);
END SpoolMem;

PROCEDURE AddToRecord(VAR R2:Line_Rec;S:ARRAY OF CHAR;Menu:CARDINAL):BOOLEAN;
(* Processes one (valid, not-empty) line of a resource file,
returns submenu or not*)

VAR T2,Ts : ARRAY[0..100] OF CHAR;

    Fl : BOOLEAN;

BEGIN
 R2.Label[0]:=CHR(0);
 ItemS(Ts,S,"|",0);
 IF Length(Ts)=0 THEN
  HALT;
  END;
 KillChar(S,CHARSET{" "});
 KillBChar(Ts,CHARSET{" "});
 IF Ts[Length(Ts)-1]="#" THEN
  Ts[Length(Ts)-1]:=CHR(0);
  END;
 Ts[20]:=CHR(0);
 Copy(R2.Name,Ts);
 ItemS(Ts,S,"|",1);
 IF Length(Ts)=0 THEN
  HALT;
  END;
 KillChar(Ts,CHARSET{" "});
 KillBChar(Ts,CHARSET{" "});
 Copy(T2,Ts);
 IF T2[0]="!" THEN
  Caps(T2);
  IF Match(T2,"*SUB*") THEN
   IF (R^.CurHigh+1)>NrO THEN
    HALT;
    END;
   R2.CallSign:=DvD+R^.CurHigh+1;
   Delete(Ts,0,4);
   KillChar(Ts,CHARSET{" "});
   Copy(R2.Label,Ts);
   RETURN TRUE;
   END;
  END;
 Copy(R2.Label,Ts);
 R2.CallSign:=R^.Labels;
 INC(R^.Labels);
 RETURN FALSE;
END AddToRecord;

PROCEDURE ReadSubMenu;
(* recursive procedure which reads a submenu*)

VAR A       : CARDINAL;
    CurP    : Line_Pnt;
    S,S2    : ARRAY[0..100] OF CHAR;
    Done    : BOOLEAN;
    AddTo   : CARDINAL;

BEGIN
  IF R^.CurHigh>NrO THEN
   HALT;
   END;
  INC(R^.CurHigh);
  Done:=FALSE;
  NEW(R^.Dat[R^.CurHigh]);
  CurP:=R^.Dat[R^.CurHigh];
  AddTo:=R^.CurHigh;
  CurP^.Next:=NIL;
  LOOP
   FIO.RdStr(F,S);
   KillChar(S,CHARSET{" "});
   INC(LineNr);
   IF Match(S,"ENDSTRUC") THEN
    EXIT
    END;
   IF FIO.EOF THEN
    EXIT
    END;
   A:=CharPos(S,";");
   IF A # MAX(CARDINAL) THEN
    Delete(S,A,Length(S)-A+1);
    END;
   KillBChar(S,CHARSET{" "});
   IF Length(S)>0 THEN
    CASE S[0] OF
     | "!" :   Delete(S,0,1);
               KillChar(S,CHARSET{" "});
               Caps(S);
               IF Match(S,"*END*") THEN
                RETURN;
               ELSE
                Error(S);
                END;
     | "$" :    IF R^.Cur # (NrO+1) THEN
                 NEW(CurP^.Next);
                 CurP:=CurP^.Next;
                 CurP^.Next:=NIL;
                 CurP^.Label:="";
                 CurP^.Name:="|";
                 CurP^.CallSign:=DvD;
                 END;
    ELSE
      IF R^.Cur # (NrO+1) THEN
       IF Done THEN
        NEW(CurP^.Next);
        CurP:=CurP^.Next;
        END;
       Done:=TRUE;
       CurP^.Next:=NIL;
       IF AddToRecord(CurP^,S,R^.CurHigh) THEN
        ReadSubMenu;
        END;
       END;
      END;
     END;
    END;
END ReadSubMenu;
(* Main module, quite similar to ReadSubMenu.*)

VAR S     : ARRAY[0..100] OF CHAR;
    S4    : ARRAY[0..3] OF CHAR;
    A     : CARDINAL;
    Done  : BOOLEAN;      (* File Created?*)
    CurP  : Line_Pnt;

BEGIN
  EnableBreakCheck;
  IF ParamCount()<1 THEN (* Parameters?*)
   HALT;
   END;
  ParamStr(S,1);
  IF NOT FIO.Exists(S) THEN (* Parameters correct?*)
   HALT
   END;
  NEW(R); Done:=FALSE;
  R^.Path:="";
  R^.First:=TRUE;
  R^.Sub:=FALSE;
  F:=FIO.Open(S);
  FIO.AssignBuffer(F,Buf);
  DisplayMem;
  LOOP
   FIO.RdStr(F,S);                   (* read line*)
   KillChar(S,CHARSET{" "});
   IF FIO.EOF THEN
    EXIT
    END;
   A:=CharPos(S,";");
   IF A # MAX(CARDINAL) THEN
    Delete(S,A,Length(S)-A+1);
    END;
   KillBChar(S,CHARSET{" "});
   IF Length(S)>0 THEN
    CASE S[0] OF
     | "@" :   IF S[1]="@" THEN
                Delete(S,0,2);
                Copy(R^.Path,S);
               ELSE
                Delete(S,0,1);
                Copy(R^.Name,S);
                END;
     | "!" :   Delete(S,0,1);
               IF NOT Done THEN
                MakeFile(R);
                Done:=TRUE;
               ELSE
                SpoolMem(F2);
                R^.First:=FALSE;
                END;
               Copy(S4,S);
               Caps(S4);
               R^.Sub:=FALSE;
               IF Compare(S4,"SUB:")=0 THEN
                Delete(S,0,4);
                R^.Sub:=TRUE;
                END;
               Copy(R^.OBJName,S);
               Init(R);
     | "*" :   IF R^.CurHigh<NrO THEN
                INC (R^.CurHigh);
                R^.Cur:=R^.CurHigh;
                NEW(R^.Dat[R^.Cur]);
                CurP:=R^.Dat[R^.Cur];
                CurP^.CallSign:=0;
                Delete(S,0,1);
                S[20]:=CHR(0);
                INC(R^.NameX);
                Copy(R^.Names[R^.NameX],S);
                R^.NameNr[R^.NameX]:=R^.Cur;
                END;
     | "$" :    IF R^.Cur # (NrO+1) THEN
                 NEW(CurP^.Next);
                 CurP:=CurP^.Next;
                 CurP^.Next:=NIL;
                 CurP^.Label[0]:=CHR(0);
                 CurP^.CallSign:=DvD;
                 CurP^.Name:="|";
                 END;
    ELSE
      IF R^.Cur # (NrO+1) THEN
       NEW(CurP^.Next);
       CurP:=CurP^.Next;
       CurP^.Next:=NIL;
       IF AddToRecord(CurP^,S,R^.Cur) THEN ReadSubMenu;END;
       END;
      END;
     END;
    END;
  IF R^.First THEN
   SpoolMem(F2);
   END;
  DisplayMem;
  FIO.WrStr(F2,"END "); FIO.WrStr(F2,R^.Name); FIO.WrChar(F2,".");
  FIO.Close(F);
  FIO.Close(F2);
END SubComp.

