MODULE CDSize;
(* A demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1996

Scan directories/files in textfile(1st parameter), and return size in bytes,
and # directories. A second parameter, if specified, should be nummeric,
and is a clustersize to be used for rounding. Used to estimate directories on
a 16k clustersize disk to a 2k clustersize CDROM, in order to get the cdrom
as full as possible.

(I also use this program to convince neighbours, friends and familymembers to
1 - partition their harddisk
2 - Use FAT32 (If they don't use kinky applications)
iow anything but 32k clusters.)

Assume you want to make a CD with directories c:\games,c:\apps and C:\modem.
An index is made with Indexer.mod, and is called c:\index.txt.

You can't include a root directory in this file, but you can include individual
names, like c:\autorun.inf. I even think these can contain wildcards.

The textfile would simply be :

c:\games
c:\apps
C:\modem
c:\index.txt

and the commandline would be CdSize textfile.txt 2048

You can also specify the clustersize in a second parameter in the textfile:

d:\
d:\ 512
d:\ 1024
d:\ 2048
d:\ 4096
d:\ 8192
d:\ 16384
d:\ 32768
d:\ 65535

Lines without clustersize specification take the clustersize specified on
the commandline if existant, else 0 (don't round to clustersize, just add)

The value 65535 is corrected to 65536 by the program.

The reason for this is that I heard WinNT can create 64k clustersize FAT16's ,
and I simply patched the program in 1 minute for a friend.

When burning a CD, watch the returned size in bytes which should be smaller
than 681 million. When a lot (>250) directories are used, subtract a megabyte
for every 100-200 directories. Some people report problems (no output, program
seems to be a nil program), I've however not been able to VID the program on
those computers.

This new version adds more intensive clustersize parameters (specify
clustersize per path), and stats per directory. *)

FROM Lib  IMPORT ParamCount,ParamStr;
FROM ELib IMPORT ExpPath,SToCard;

FROM Str IMPORT Concat,Length,ItemS;
IMPORT IO,FIO,Crt,DirTree;

CONST CR=CHR(13)+CHR(10);

      Usage=CR+CR+

 "Computes the total size of directories+files, stored in INFILE (ASCII)."+CR+
 "The second optional parameter is the clustersize in bytes, default 2048."+CR+
 "Usage : CDSIZE <INFILE> [clustersize]"+ CR+
 CR+
 "      E.g. CDSIZE c:\newcd.txt 4096"+CR+CR;

PROCEDURE ShowRecord(Dir:FIO.PathStr;Fil:FIO.DirEntry);
END ShowRecord;

VAR Param1,Param2,
    Par21,Par22  : ARRAY[0..127] OF CHAR; (* Size string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)

                (* Par2x is result of ItemS(Par2x,Param2,x-1," "); *)

    Fl            : BOOLEAN;
    Bytes         : LONGCARD;
    Files,Dirs    : CARDINAL;
    F             : FIO.File;
    D             : FIO.DirEntry;
    DefClus       : CARDINAL;

BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("CDSIZE 1.0, EXE is PD, source is (C) Marco van de Voort.");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn; IO.WrLn;

 IF ParamCount() <1 THEN
   IO.WrStr(Usage);
   HALT;
   END;
 ParamStr(Param1,1);
 IF ParamCount()=2 THEN
  ParamStr(Param2,2);
  DefClus:=SToCard(Param2,Fl);
  DirTree.ClusterSize:=DefClus;
  IF NOT Fl THEN
   HALT;
   END;
  END;
 IF NOT FIO.Exists(Param1) THEN
  Crt.TextColor(Crt.LightRed);
  IO.WrStr("Error : File doesn't exist."); IO.WrLn;
  HALT
  END;
 Files:=0; Bytes:=0; Dirs:=0; DirTree.DirsToo:=FALSE;
 Crt.CrtOff; (*Pipable *)
 F:=FIO.Open(Param1);
 LOOP
  FIO.RdStr(F,Param2);
  ItemS(Par21,Param2," ",0);
  IF Par21[0]#";" THEN
   ItemS(Par22,Param2," ",1);
   IF Par22[0]=CHR(0) THEN
    DirTree.ClusterSize:=DefClus;
   ELSE
    DirTree.ClusterSize:=SToCard(Par22,Fl);
    END;
   IF FIO.EOF  THEN
    EXIT
    END;
   IO.WrStr(Par21);
   Fl:=ExpPath(Par21); (* Simplify the path, ..\ .\ etc are expanded
                          makes scan a lot more robust *)
   Fl:=FIO.ReadFirstEntry(Par21,FIO.FileAttr{FIO.hidden,FIO.readonly,FIO.archive,FIO.system,FIO.directory},D);
   IF Length(Par21)<4 THEN
    DirTree.FileScan(Par21,"*.*",ShowRecord);
    INC(Files,DirTree.FoundCount);
    INC(Bytes,DirTree.TotalBytes);
    INC(Dirs, DirTree.FoundDirs); (* Patch for root problem*)
    END;
   IF Fl THEN
   IF FIO.directory IN D.attr THEN
    DirTree.FileScan(Par21,"*.*",ShowRecord);
    INC(Files,DirTree.FoundCount);
    INC(Bytes,DirTree.TotalBytes);
    INC(Dirs, DirTree.FoundDirs);
   ELSE
    INC (Files);
    INC (Bytes,D.size);
    END;
 
    END;
    IO.WrStr(" C. siz:"); IO.WrCard(DirTree.ClusterSize,5); IO.WrChar(" ");
    IO.WrLngCard(DirTree.TotalBytes,15); IO.WrStr(" bytes in ");
    IO.WrCard(DirTree.FoundCount,5); IO.WrStr(" files and ");
    IO.WrCard(DirTree.FoundDirs,5); IO.WrStr(" directories."); IO.WrLn;
    END;
  END;
 FIO.Close(F);
 IO.WrLn;
 IO.WrLngCard(Bytes,0); IO.WrStr(" bytes in ");
 IO.WrCard(Files,0); IO.WrStr(" files and ");
 IO.WrCard(Dirs,0); IO.WrStr(" directories.");
 IO.WrLn; IO.WrLn;
END CDSize.