MODULE FindArchives;
(* A demonstration of module DirTree, XtdLib package,
                              (C) by Marco van de Voort 1996

This module is NEW in 1.04. Meant to demonstrate the revised (procedure
variable) features of SearchForFiles.

The BuildTree/(multiple) SearchForFiles/ ScanFile routines are extremely
powerfull.

The program tries to identify all files in the directory given as 1st (and
only) parameter. It'll display the files which are archives according to
ELib.ArchiveMethod. (a.o. ZIP ARJ RAR etc)

It will probably not work on your C drive if you're running Win95, since the
locking fails under Win95 as far as the swapfile is concerned. All other
files seem to work.

Not limited to archives. Ideal for scanning for masqueraded garbadge in
user (mail)directories.

*)

FROM Str     IMPORT Copy,Append,Length;
FROM DirTree IMPORT BuildTree,DirTreePoint,FilePtr,SearchForFiles,ScanTree;
FROM Storage IMPORT ALLOCATE,DEALLOCATE,HeapAvail,MainHeap;
FROM Lib     IMPORT ParamCount,ParamStr;
FROM ELib    IMPORT ExpPath,ArchiveMethod;
FROM Multi   IMPORT LockFile;

IMPORT IO,FIO,Crt,DirTree;

PROCEDURE DisplayFile(S:ARRAY OF CHAR;DirE:FIO.DirEntry);

BEGIN
 IO.WrStr(S); IO.WrStr(DirE.Name); IO.WrLn;
END DisplayFile;

PROCEDURE Usage;

BEGIN
 IO.WrStr("Finds all archives detectable with ELib.ArchiveMethod in specified dir");
 IO.WrLn;
 IO.WrStr("and below. Uses a file-tree-structure in memory, outputs data afterwards");
 IO.WrLn;
 IO.WrStr("Usage : FindArch <Directory>"); IO.WrLn;
 IO.WrStr("      E.g. FindArch c:\ a");
 IO.WrLn; IO.WrLn;
END Usage;

PROCEDURE IdentArchive(D:FIO.DirEntry;Path : ARRAY OF CHAR):BOOLEAN;

VAR S :ARRAY[0..255] OF CHAR;
    A : CARDINAL;
    F : FIO.File;

BEGIN
 Copy(S,Path);
 Append(S,D.Name);   (*IO.WrStr(S);

 F:=FIO.OpenRead(S);  IO.WrCard(LockFile(F),5); FIO.Close(F); IO.WrLn; *)
 A:=CARDINAL(ArchiveMethod(S));
 (* IF A=15 THEN
     A:=DoOtherdetection(S);  Let A be 15 if no fileformat detected
     END; *)
 RETURN A#15;
END IdentArchive;

VAR Root : DirTreePoint;
    Param1: ARRAY[0..127] OF CHAR; (* Size string is important,
                                               see notes about ExpPath in
                                               ELib.DEF or in the Help *)
    Fl            : BOOLEAN;
    A             : CARDINAL;
    Files,Dirs    : CARDINAL;
    Bytes:LONGCARD;
    MemStartUp    : LONGCARD;
BEGIN
 Crt.TextColor(Crt.Yellow);
 IO.WrStr("FindArch 1.0, (C) Marco van de Voort, XtdLib package");
 Crt.TextColor(Crt.LightGray);
 IO.WrLn; IO.WrLn;

 IF ParamCount() <1 THEN
   Usage;
   HALT;
   END;
 ParamStr(Param1,1);
 IF NOT FIO.Exists(Param1) THEN
  Crt.TextColor(Crt.LightRed);
  IO.WrStr("Error : Path doesn't exist."); IO.WrLn;
  HALT
  END;
 Crt.CrtOff;
 MemStartUp:=LONGCARD(HeapAvail(MainHeap))*16;
 DirTree.DirsToo:=TRUE;
 Fl:=ExpPath(Param1); (* Simplify the path, ..\ .\ etc are expanded,
                         makes scan a lot more robust, ExpPath has
                         however some theoretical drawbacks, see ELib.DEF.*)
 Crt.CrtOn;
 Root:=BuildTree(Param1);                (* Build Directory tree in memory
                                            (SO ONLY DIRECTORIES!) *)
 Files:=0; Bytes:=0; Dirs:=0;
 FIO.ShareMode:=FIO.ShareNoInherit;


 SearchForFiles(Root,"*.*",IdentArchive);          (* add files selectively (e.g. *.mod) to
                                           the tree*)
 Crt.CrtOff; (* Next output should be pipable, normal output*)

 INC(Files,DirTree.FoundCount);
 INC(Bytes,DirTree.TotalBytes);

 ScanTree(Root,DisplayFile,NULLPROC);              (* Display found files with complete
                                                   path*)
 MemStartUp:=MemStartUp-(LONGCARD(HeapAvail(MainHeap))*16);
 DirTree.KillFileTree(Root);


 IO.WrCharRep("-",78);
 IO.WrLn;
 IO.WrCard(Files,25); IO.WrStr(" files, ");
 IO.WrLngCard(Bytes,12); IO.WrStr(" bytes in ");
 IO.WrCard(DirTree.FoundDirs,0); IO.WrStr(" directories");
 IO.WrLn; IO.WrLn;
 IO.WrStr("Files and directories occupied "); IO.WrLngCard(MemStartUp,0); IO.WrStr(" bytes of memory");
 IO.WrLn;
END FindArchives.
