MODULE PlayVoc;
(* VOC player, initial version, packaged in version 0.95 the last minute.

Plays voc file on commandline, in normal ( = not DMA) mode. Byte for byte,
WrBlkDSP not ready yet. Part of the XtdLib package(C) 1997. *)

FROM Lib IMPORT NoSound,ParamStr,ParamCount;
IMPORT FIO,IO,Storage;
FROM CoreMusic IMPORT ResetSoundBl,WrDSP,SBPort,SpeakerOn,
                      SpeakerOff,WrDSPRaw,VocHdr,VocDataHdr,ShutUpAdlib;
FROM ELib IMPORT Hi;

CONST
   resetport  = 0226H;
   readport   = 022AH;
   Writeport  = 022CH;
   statusport = 022EH;
   dac_Write  = 010H;
   adc_read   = 020H;
   midi_read  = 030H;
   midi_Write = 038H;
   speakeron  = 0D1H;
   speakeroff = 0D3H;

PROCEDURE ReadAndOutputVocHdr(F:FIO.File):LONGCARD;

VAR Hdr  : VocHdr;
    DHdr : VocDataHdr;
    A    : CARDINAL;
    Len  : LONGCARD;
    Freq : LONGCARD;

BEGIN
 A:=FIO.RdBin(F,Hdr,SIZE(VocHdr));
 A:=FIO.RdBin(F,DHdr,SIZE(VocDataHdr));

 IO.WrStrAdj("Signature :",-30);
 IO.WrStr(Hdr.Signature); IO.WrLn;
 IO.WrStrAdj("Data start:",-30);
 IO.WrCard(Hdr.Data_start,5); IO.WrLn;
 IO.WrStrAdj("Version driver req'd:",-30);
 IO.WrCard(Hdr.Version>>8,2); IO.WrChar(".");
 IO.WrShtCard(BYTE(Hdr.Version),0);
 IF ((65535-Hdr.Version) +01234H) =Hdr.id THEN
  IO.WrStr("      Voc version checked");
 ELSE
  IO.WrStr("      Incorrect VOC format");
  END;
  IO.WrLn;
 IO.WrStrAdj("VOC-ID:",-30);
 IO.WrCard(Hdr.id,5); IO.WrLn;

 Len:=LONGCARD(ORD(DHdr.len[1])+ORD(DHdr.len[2])*256)+LONGCARD(ORD(DHdr.len[3])*256)*256;
 IO.WrLn; IO.WrStr("Sample header"); IO.WrLn;
 IO.WrStrAdj("sample id:",-30);
 IO.WrShtCard(DHdr.id,5); IO.WrStr(" should be 1 "); IO.WrLn;
 IO.WrStrAdj("Sampledata length:",-30);
 IO.WrLngCard(Len,12); IO.WrLn;
 IO.WrStrAdj("Samplerate :",-30);
 IO.WrShtCard(DHdr.sr,5);
 Freq:=LONGCARD(1000000 DIV LONGCARD(256 - CARDINAL(DHdr.sr)));
 IO.WrStr(" = "); IO.WrLngCard(Freq,12); IO.WrStr("Hz"); IO.WrLn;
 IO.WrStrAdj("Pack method :",-30); IO.WrShtCard(DHdr.pack,5); IO.WrStr(" = ");
 CASE DHdr.pack OF
  | 0: IO.WrStr("unpacked");
  | 1: IO.WrStr("4-bit");
  | 2: IO.WrStr("2.6 bit");
  | 3: IO.WrStr("2 bit packed");
  END;
  IO.WrLn;
 RETURN Len;
END ReadAndOutputVocHdr;

TYPE BufferT= POINTER TO ARRAY[0..65534] OF BYTE;

PROCEDURE loadFile(VAR buffer:BufferT; Filename:ARRAY OF CHAR) : CARDINAL;
(* Load a file and display the .VOC header *)

VAR
   fromf : FIO.File;
   size : LONGCARD;
   errcode : CARDINAL;
BEGIN
   IF FIO.Exists(Filename) THEN
    fromf:=FIO.Open(Filename);
    size:=ReadAndOutputVocHdr(fromf);
    Storage.ALLOCATE(buffer,CARDINAL(size));
    errcode:=FIO.RdBin(fromf,buffer^,CARDINAL(size));
    FIO.Close(fromf);
    RETURN errcode;
   ELSE
    RETURN 0
    END;
END loadFile;

PROCEDURE unload(buffer:BufferT; size:CARDINAL);
BEGIN
   Storage.DEALLOCATE(buffer,size);
END unload;

VAR
   ch    : CHAR;
   buf   : BufferT;
   index,
   fsize : CARDINAL;
   S     : ARRAY[0..99] OF CHAR;

BEGIN
 IF ParamCount()<1 THEN
  IO.WrStr("Usage : Playvoc [VOCFILE]"); IO.WrLn; IO.WrLn;
  HALT;
  END;
 ParamStr(S,1);
 IF SBPort=0 THEN
  IO.WrStr("No soundblaster found");
  HALT;
 ELSE
  IO.WrStr("Sound blaster detected on port :");
  IO.WrHex(SBPort,5);
  IO.WrChar("H");
  IO.WrLn;
  END;
 IF NOT ResetSoundBl() THEN
  IO.WrStr('Unable TO initialize SoundBlaster.'); IO.WrLn;
  HALT;
  END;
 fsize := loadFile(buf,S);
 IF (fsize <= 0) THEN
  IO.WrStr(S); IO.WrStr(' NOT found.'); IO.WrLn;
  HALT;
  END;
   SpeakerOn;
   IO.WrStr('Playing...'); IO.WrLn;

   FOR index:=0 TO fsize DO  (* The real sample loop, write a byte to the DSP
                                 with a delay of '50',
                                 the delay can be increased for faster
                                 computers (50=P166+) or decrease for slower
                                 computers (36 is 486-66), dependant also
                                 of the sample rate.*)
    WrDSP(75,buf^[index]);
    END;
   SpeakerOff;
   unload(buf,fsize);
   IO.WrStr('Done, press a key.'); IO.WrLn;
   ch := IO.RdKey();
   ShutUpAdlib;   NoSound;  (* Shut-up all channels, speakeroff should be
                                enough, just in case *)
END PlayVoc.