MODULE SegCount;
(* Util belonging to the XtdLib package, (C) by Marco van de Voort 1996

When you compile a module, you can see some information about its size (code,
variables and constants). This util displays similar information calculated
out of the disassembly made by tsda.exe .

Mainly written to look what all those processor optimalisations brought up
in bytes. (Also speed of course, but that's somewhat more difficult to measure)

One thing which is not standard is that the compiler when compiling for xlarge
(or compatible) puts constants in the codesegment. This looks like code, but
is a constant.

Use : SegCount <disassembly>
*)

FROM Str        IMPORT Append,Delete,Length,Match,CharPos,Slice,StrToCard,Compare,Copy;
FROM Storage    IMPORT ALLOCATE;
FROM Lib        IMPORT Fill,ParamStr,ParamCount;
IMPORT FIO,IO;

TYPE SegmentP   = POINTER TO Segment;
     Segment    =     RECORD
                       Next     : SegmentP;
                       Name1,
                       Name2    : ARRAY[0..99] OF CHAR;
                       Size     : LONGCARD;
                       SegNr    : CARDINAL;
                       END;

VAR     Root : SegmentP;
        First: BOOLEAN;

PROCEDURE SearchTree(Line : ARRAY OF CHAR);

VAR  Name1,Name2 : ARRAY[0..99] OF CHAR;
     Size    : LONGCARD;
     SegNr   : CARDINAL;
     LeftP,
     RightP,
     Comma,
     Equal   : CARDINAL;
     CurP    : SegmentP;
     Fl      : BOOLEAN;

BEGIN
 Delete(Line,0,8);              (* Delete "segment " text *)

 LeftP :=CharPos(Line,"(");     (* calculate some positions *)
 RightP:=CharPos(Line,")");
 Comma :=CharPos(Line,",");
 Equal :=CharPos(Line,"=");

 Slice(Name1,Line,Equal+2,5);   (* Calculate the two numbers*)
 Size:=StrToCard(Name1,16,Fl);
 Slice(Name1,Line,Comma+1,5);
 SegNr:=CARDINAL(StrToCard(Name1,16,Fl));

 Slice(Name1,Line,0,LeftP);      (* save the two strings*)
 Slice(Name2,Line,LeftP+1,Comma-1-LeftP);

 Fl:=FALSE;
 IF First THEN
  First:=FALSE;
  NEW(Root);
  CurP:=Root;
  Fill(ADR(CurP^),SIZE(Segment),0);
  Copy(CurP^.Name1,Name1);
  Copy(CurP^.Name2,Name2);
  CurP^.Size:=Size;
  CurP^.SegNr:=SegNr;
 ELSE
  CurP:=Root;
  LOOP
   IF Compare(Name1,CurP^.Name1)=0 THEN
    Fl:=TRUE;
    INC(CurP^.Size,Size);
    END;
   IF CurP^.Next=NIL THEN
    EXIT
    END;
   CurP:=CurP^.Next;
   END;
   IF NOT Fl THEN
    NEW(CurP^.Next);
    CurP:=CurP^.Next;
    Fill(ADR(CurP^),SIZE(Segment),0);
    Copy(CurP^.Name1,Name1);
    Copy(CurP^.Name2,Name2);
    CurP^.Size:=Size;
    CurP^.SegNr:=SegNr;
    END;
  END;
END SearchTree;

PROCEDURE Output;

VAR CurP        : SegmentP;

BEGIN
 IF NOT First THEN
  IO.WrStr(" SegmentNr  SegName                       GroupName        Size ");
  IO.WrLn;
  IO.WrCharRep("-",79); IO.WrLn;
  CurP:=Root;
  LOOP
   IO.WrHex(CurP^.SegNr,4);   IO.WrChar("H");
   IO.WrCharRep(" ",7);
   IO.WrStrAdj(CurP^.Name1,-30);
   IO.WrStrAdj(CurP^.Name2,-10);
   IO.WrLngCard(CurP^.Size,11);
   IO.WrLn;
   IF CurP^.Next=NIL THEN
    EXIT
    END;
   CurP:=CurP^.Next;
   END;
  END;
END Output;


VAR  F        : FIO.File;               (* Filehandle inputfile*)
     B        : ARRAY[0..4095] OF BYTE; (* Buffer inputfile *)
     Read     : ARRAY[0..99] OF CHAR;
     S        : ARRAY[0..99] OF CHAR;

BEGIN
 IF ParamCount()<1 THEN
  IO.WrStr(" SegCount, add up all segments in a disassembly");IO.WrLn;IO.WrLn;
  IO.WrStr("Usage : SegCount <disassembled module>"); IO.WrLn;
  HALT;
  END;
 First:=TRUE;
 ParamStr(S,1);
 IF NOT FIO.Exists(S) THEN
  Append(S,".DA");
  IF NOT FIO.Exists(S) THEN
   HALT;
   END;
  END;
 F:=FIO.Open(S);
 FIO.AssignBuffer(F,B);
 LOOP
  FIO.RdStr(F,S);
  IF FIO.EOF THEN
   EXIT
   END;
  IF Match(S,"*segment length*") THEN (* detecting this is more reliable than
                                          "segment" alone *)
   SearchTree(S);
   END;
  END;
 FIO.Close(F);
 Output;
END SegCount.