IMPLEMENTATION MODULE CRC32;

(* Modula-2 CRC-32 calculation.

   written by : Andreas Reinicke, 2:284/402@fidonet.org
   version    : 1.00 (January 7, 1994)
*)

TYPE
   CalcBlock = RECORD
      CASE : CARDINAL OF
      | 0 : OldCRC    : LONGCARD;
            Zero      : SHORTCARD;
      | 1 : Index     : SHORTCARD;
            TempCRClo : BITSET;
            TempCRChi : BITSET;
      | 2 : dummy     : SHORTCARD;
            NewCRC    : LONGCARD;
      END;
   END;

   CRCtabType = RECORD
      CASE : CARDINAL OF
      | 0 : elem : LONGCARD;
      | 1 : lo   : BITSET;
            hi   : BITSET;
      END;
   END;

VAR
   CRCtab: ARRAY [0..255] OF CRCtabType;

PROCEDURE CreateCRC32Table( PolyHi, PolyLo : BITSET );
VAR
   I, J   : CARDINAL;
   TempHi,
   TempLo : CARDINAL;
   DoXOR  : BOOLEAN;
BEGIN
   FOR I := 0 TO 255 DO
      TempHi := 0;
      TempLo := I;
      FOR J := 1 TO 8 DO
         IF (TempLo MOD 2 = 1) THEN
            DoXOR := TRUE;
         ELSE
            DoXOR := FALSE;
         END;
         TempLo := TempLo >>1;
         IF (TempHi MOD 2 = 1) THEN
            TempLo := TempLo+08000H;
         END;
         TempHi := TempHi >>1;
         IF DoXOR THEN
            TempHi := CARDINAL(BITSET(TempHi)/PolyHi);
            TempLo := CARDINAL(BITSET(TempLo)/PolyLo);
         END;
      END;
      CRCtab[I].hi := BITSET(TempHi);
      CRCtab[I].lo := BITSET(TempLo);
   END;
END CreateCRC32Table;

PROCEDURE CalcCRC( crc     : LONGCARD;
                   BufAdr  : ADDRESS;
                   BufSize : CARDINAL ) : LONGCARD;
VAR
   segm, offs, I : CARDINAL;
BEGIN
   segm := SYSTEM.Seg( BufAdr^ );
   offs := SYSTEM.Ofs( BufAdr^ );
   FOR I := 0 TO BufSize-1 DO
      AddCRC( crc, BYTE( [segm:offs+I]^ ) );
   END;
   RETURN crc;
END CalcCRC;

PROCEDURE AddCRC( VAR crc : LONGCARD; ch : SYSTEM.BYTE );
VAR
   Calc : CalcBlock;
BEGIN
   Calc.OldCRC := crc;
   Calc.Zero   := 0;
   Calc.Index  := SHORTCARD(BITSET(Calc.Index) / BITSET(ch));
   Calc.TempCRClo := Calc.TempCRClo / CRCtab[CARDINAL(Calc.Index)].lo;
   Calc.TempCRChi := Calc.TempCRChi / CRCtab[CARDINAL(Calc.Index)].hi;
   crc := Calc.NewCRC;
END AddCRC;

PROCEDURE InvertCRC( VAR crc : LONGCARD );
VAR
   crc32 : RECORD
      CASE : CARDINAL OF
      | 0 : elem : LONGCARD;
      | 1 : lo   : BITSET;
            hi   : BITSET;
      END;
   END;
BEGIN
   crc32.elem := crc;
   crc32.lo   := crc32.lo / BITSET(0FFFFH);
   crc32.hi   := crc32.hi / BITSET(0FFFFH);
   crc        := crc32.elem;
END InvertCRC;

BEGIN
END CRC32.
